/*
    $Id: client.c 292 2010-02-14 07:37:34Z irfangul82 $
    Simple file and directory encryption
    Copyright (C) 2010  Irfan Gulamali

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifdef CLIENT
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if OS == 0x03 //Linux OS
#include <termios.h>
#include <unistd.h>
#endif

#if OS == 0x02 //Windows OS
#include <windows.h>
#include <time.h>
#endif

#include "client.h"
#include "util.h"
#include "glob.h"

unsigned char host[120];
unsigned char fl_dir[PATH_MAX];
unsigned char port[5] = "8080";

int main(int argc, char **argv)
{
    int request = 0;
    int status = 0;
    int cntr = 0;
    unsigned char key[34]; //see fgets -size needs to be 34
    unsigned char hash[41];
    unsigned char *pars_fl = 0;

    memset(hash, 0, sizeof(hash));
    memset(key, 0, sizeof(key));
    request = parseParams(argc, argv);

    if (request)
    {
        switch (request)
        {
        case ENCFL:
            parseFileName(fl_dir, &pars_fl);
            getPass(key);
            status = encryptFile(pars_fl, key);
            memset(key, 0, 34);
            if (status)
            {
                procErr(status);
                free(pars_fl);
                return status;
            }
            fprintf(stdout, "\nDone.\n");
            free(pars_fl);
            return 0;

        case DECFL:
            parseFileName(fl_dir, &pars_fl);
            getPass(key);
            status = decryptFile(pars_fl, key);
            memset(key, 0, 34);
            if (status)
            {
                procErr(status);
                free(pars_fl);
                return status;
            }
            fprintf(stdout, "\nDone.\n");
            free(pars_fl);
            return 0;

        case ENCDIR:
            parseFileName(fl_dir, &pars_fl);
            getPass(key);
            fprintf(stdout, "\nEncrypting directory...");
            status = encryptDir(pars_fl, key);
            memset(key, 0, 34);
            if (status)
            {
                procErr(status);
                free(pars_fl);
                return status;
            }
            fprintf(stdout, "\nDone.\n");
            free(pars_fl);
            return 0;

        case DECDIR:
            parseFileName(fl_dir, &pars_fl);
            getPass(key);
            fprintf(stdout, "\nDecrypting directory...");
            status = decryptDir(pars_fl, key);
            memset(key, 0, 34);
            if (status)
            {
                procErr(status);
                free(pars_fl);
                return status;
            }
            fprintf(stdout, "\nDone.\n");
            free(pars_fl);
            return 0;

        case GTSHA:
            parseFileName(fl_dir, &pars_fl);
            status = gtHash(pars_fl, hash);
            if (status)
            {
                procErr(status);
                free(pars_fl);
                return status;
            }
            free(pars_fl);
            while (cntr < 40)
            {
                fprintf(stdout,"%c", hash[cntr]);
                cntr++;
            }
            fprintf(stdout, " *%s\n", fl_dir);
            return 0;

        case GTDIRSHA:
            parseFileName(fl_dir, &pars_fl);
            status = gtHashDir(pars_fl);
            if (status)
            {
                procErr(status);
                free(pars_fl);
                return status;
            }
            free(pars_fl);
            return 0;

        case ENCMSG:
            parseFileName(fl_dir, &pars_fl);
            getPass(key);
            fprintf(stdout, "\nEncrypting message...");
            status = encMsg(pars_fl, key);
            memset(key, 0, 34);
            if (status)
            {
                procErr(status);
                free(pars_fl);
                return status;
            }
            free(pars_fl);
            fprintf(stdout, "\nDone.\n");
            return 0;

        case DECMSG:
            parseFileName(fl_dir, &pars_fl);
            getPass(key);
            fprintf(stdout,"\nDecrypting message...\n");
            status = decMsg(pars_fl, key);
            memset(key, 0, 34);
            if (status)
            {
                procErr(status);
                free(pars_fl);
                return status;
            }
            free(pars_fl);
            fprintf(stdout, "\nDone.\n");
            return 0;

        case 0:
            return 0;
        }
    }
    else
    {
        usage();
        return -1;
    }
}

void usage()
{
char gplText[] = "\
Kokla - Simple file and directory encryption\n \
http://sourceforge.net/projects/kokla/\n \
Copyright (C) 2010  Irfan Gulamali\n\n \
This program is free software: you can redistribute it and/or modify\n \
it under the terms of the GNU General Public License as published by\n \
the Free Software Foundation, either version 3 of the License, or\n \
(at your option) any later version.\n\n \
This program is distributed in the hope that it will be useful,\n \
but WITHOUT ANY WARRANTY; without even the implied warranty of\n \
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n \
GNU General Public License for more details.\n\n \
You should have received a copy of the GNU General Public License\n \
along with this program.  If not, see <http://www.gnu.org/licenses/>.";

  printf("%s", gplText);
    printf("\n[%s: Build Date: %s] \n", VERSION, __DATE__);
    printf("http://code.google.com/p/kokla/\n");
    printf("\nUsage:\t kokla [options]... [-f|-D] [FILE|DIRECTORY]");
    printf("\nUsage:\t kokla -em FILE");
    printf("\nUsage:\t kokla -dm FILE");
    printf("\n");
    printf("\nMain operation mode:");
    printf("\n -e,\t encrypt mode (AES-256)");
    printf("\n -d,\t dencrypt mode");
    printf("\n -H,\t hash mode (SHA-1)");
    printf("\n");
    printf("\nLocal file selection:");
    printf("\n -f,\t specify file to operate on");
    printf("\n -D,\t specify directory to operate on");
    printf("\n");
    printf("\nSub operation mode:");
    printf("\nEncrypt/Encode small messages (file < 5MB) as base64");
    printf("\n -em,\t specify file to encrypt and encode as base64 message");
    printf("\n -dm,\t specify file to decrypt and encode from base64 message\n");

}

void procErr(int err)
{
    switch (err)
    {
    case FILEIO_ERROR:
        fprintf(stderr, "\nA file IO error occurred -- check file path!\n");
        break;
    case FILEFORMAT_ERROR:
        fprintf(stderr, "\nFile format error\n");
        break;
    case FILESZ_OUTBOUND:
        fprintf(stderr, "\nFile size out of range see documentation\n");
        break;
    case NET_ERR:
        fprintf(stderr, "\nA network error was encountered!\n");
        break;
    case BAD_NET:
        fprintf(stderr, "\nBad network information was provided!\n");
        break;
    case CON_CLOSE:
        fprintf(stderr, "\nThe connection was closed!\n");
        break;
    case CON_TIMEOUT:
        fprintf(stderr, "\nThe connection timed out\n");
        break;
    case BAD_PASS:
        fprintf(stderr, "\nIncorrect password!\n");
        break;
    case SHA_ERR:
        fprintf(stderr, "\nExpected SHA-1 hash failed!\n");
        break;
    default:
        fprintf(stderr, "\nUnknown Error!\n");
        break;
    }
}

/* Possible inputs to kokla client:
   /-------- DEPRECATED --------------
   k -h host -p port -D dir
   k -h host -p port -f file
   k -p port -h host -D dir
   k -p port -h host -f file
   k -h host -D dir
   k -h host -f file
   /---------- DEPRECATED ------------
   k -e -f file
   k -e -D dir
   k -d -f file
   k -d -D dir
   k -H -f file
   k -H -D dir

   k -em file
   k -dm file

*/

int parseParams(int argc, char **argv)
{

    unsigned short int cntr = 0;

    /*
      strip data from parameters
    */
    while ( cntr < argc)
    {
        //removed network related options
        /* if (!strcmp(argv[cntr], "-h") && (cntr+1 < argc))
        strncpy(host, argv[cntr+1], strlen(argv[cntr+1]));

        if (!strcmp(argv[cntr], "-p") && (cntr+1 < argc))
        strncpy(port, argv[cntr+1], strlen(argv[cntr+1]));
        */
        if (!strcmp(argv[cntr], "-f") && (cntr+1 < argc))
            strncpy(fl_dir, argv[cntr+1], strlen(argv[cntr+1]));

        if (!strcmp(argv[cntr], "-D") && (cntr+1 < argc))
            strncpy(fl_dir, argv[cntr+1], strlen(argv[cntr+1]));

        if (!strcmp(argv[cntr], "-em") && (cntr+1 < argc))
            strncpy(fl_dir, argv[cntr+1], strlen(argv[cntr+1]));

        if (!strcmp(argv[cntr], "-dm") && (cntr+1 < argc))
            strncpy(fl_dir, argv[cntr+1], strlen(argv[cntr+1]));


        cntr++;
    }

    if ( argc==3)
    {
        if (!strcmp(argv[1], "-em"))
            return ENCMSG;

        if (!strcmp(argv[1], "-dm"))
            return DECMSG;
    }

    if ( argc==4)
    {
        if (!strcmp(argv[1], "-e"))
            if (!strcmp(argv[2],"-f"))
                return ENCFL;


        if (!strcmp(argv[1], "-e"))
            if (!strcmp(argv[2],"-D"))
                return ENCDIR;


        if (!strcmp(argv[1], "-d"))
            if (!strcmp(argv[2],"-f"))
                return DECFL;


        if (!strcmp(argv[1], "-d"))
            if (!strcmp(argv[2],"-D"))
                return DECDIR;

        if (!strcmp(argv[1], "-H"))
            if (!strcmp(argv[2],"-D"))
                return GTDIRSHA;

        if (!strcmp(argv[1], "-H"))
            if (!strcmp(argv[2],"-f"))
                return GTSHA;

    }


    return 0;
}


int sndFile(char *fname, char *host, int port)
{

    return 0;
}

int sndDir(char *fname, char *host, int port)
{

    return 0;
}

void getPass(char *pass)
{
#if OS == 0x03 //Linux OS
    struct termios defrsett, newrsett;
    tcgetattr(fileno(stdin), &defrsett);
    newrsett = defrsett;
    newrsett.c_lflag &= ~ECHO;
    tcsetattr(fileno(stdin), TCSAFLUSH, &newrsett);

    int cntr = 0;
    unsigned char verifyPass[34];
    memset(verifyPass, 0, sizeof(verifyPass));
    memset(pass, 0, 32);
    memset(verifyPass, 1, 32);
    while (strcmp(pass, verifyPass))
    {
        memset(pass, 0, 32);
        memset(verifyPass, 0, 32);
        printf("\npassword: ");
        fgets(pass, 33, stdin);
        printf("\nconfirm: ");
        fgets(verifyPass, 33, stdin);
    }


    while (cntr < 32)
    {
        if (pass[cntr] == '\n')
        {
            pass[cntr] = '\0';
        }
        cntr++;
    }

    tcsetattr(fileno(stdin), TCSANOW, &defrsett);
#endif


#if OS == 0x02 //windows
    int cntr = 0;
    char ch;
    unsigned char verifyPass[34];
    memset(verifyPass, 0, sizeof(verifyPass));
    memset(pass, 0, 32);
    memset(verifyPass, 1, 32);
    while (strcmp(pass, verifyPass))
    {
        memset(pass, 0, 32);
        memset(verifyPass, 0, 32);
        cntr = 0;
        ch = '\0';
        printf("\npassword: ");

        while (cntr < 32 && ch != '\n' && ch != '\r')
        {
            ch = _getch();
            pass[cntr] = ch;
            cntr++;
        }
        ch = '\0';
        pass[cntr] = ch;
        cntr = 0;
        printf("\nconfirm: ");
        while (cntr < 32 && ch != '\n' && ch != '\r')
        {
            ch = _getch();
            verifyPass[cntr] = ch;
            cntr++;
        }
        ch = '\0';
        verifyPass[cntr] = ch;
    }

    cntr = 0;
    while (cntr < 32)
    {
        if ((pass[cntr] == '\n') || (pass[cntr] == '\r'))
        {
            pass[cntr] = '\0';
        }
        cntr++;
    }
#endif
}
#endif

