/*
    $Id: mutex.c 292 2010-02-14 07:37:34Z irfangul82 $
    Simple file and directory encryption
    Copyright (C) 2010  Irfan Gulamali

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <stdlib.h>

/* Linux Mutex implementation */
#if OS == 0x03

#include <pthread.h>

void createMTX(void **ptr)
{
    *ptr = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
}

void holdMTX(void **ptr)
{
    pthread_mutex_lock((pthread_mutex_t *)(*ptr));
}

void releaseMTX(void **ptr)
{
    pthread_mutex_unlock((pthread_mutex_t *)(*ptr));
}

void cleanMTX(void **ptr)
{
    free(*ptr);
}
#endif

/* WIN Mutex implementation */
#if OS == 0x02

#include <windows.h>

void createMTX(void **ptr)
{
    HANDLE tmp;
    tmp = CreateMutex(NULL, 0, NULL);
    *ptr = malloc(sizeof(HANDLE));
    memcpy(*ptr, &tmp, sizeof(HANDLE));
}

void holdMTX(void **ptr)
{
    WaitForSingleObject(*((HANDLE*)(*ptr)), INFINITE);
}

void releaseMTX(void **ptr)
{
    ReleaseMutex(*((HANDLE*)(*ptr)));
}

void cleanMTX(void **ptr)
{
    free(*ptr);
}
#endif
