/*
    $Id: net.c 292 2010-02-14 07:37:34Z irfangul82 $
    Simple file and directory encryption
    Copyright (C) 2010  Irfan Gulamali

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "glob.h"
#include "net.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


int netInit(net_ctx **ctx, unsigned char host[255], int port, int sock_fam,
            int sock_type, int sock_proto)
{
    struct hostent *hst;
    int netRet = 0;
    (*ctx) = (net_ctx *) malloc(sizeof(net_ctx));

#if OS == 0x02
    WSADATA wsaData;
    WORD wVersionRequested = MAKEWORD(2, 2);
    netRet = WSAStartup(wVersionRequested, &wsaData);
#endif

    /* setup windows network connection	 */
#if OS == 0x02 //windows
    if (netRet)
    {
        return NET_ERR;
    }

    (*ctx)->sock = socket(sock_fam, sock_type, sock_proto);
    if ((*ctx)->sock == INVALID_SOCKET)
    {
        WSACleanup();
        return NET_ERR;
    }
    hst = gethostbyname(host);
    if (hst)
    {
        memcpy(&(((*ctx)->sa).sin_addr), hst->h_addr_list[0], hst->h_length);

    }
    else
    {
        (*ctx)->sa.sin_addr.S_un.S_addr = inet_addr(host);
    }

    (*ctx)->sa.sin_family = sock_fam;
    (*ctx)->sa.sin_port = htons(port);

#endif

    /* setup linux network connection	 */
#if OS == 0x03
    (*ctx)->sock = socket(sock_fam, sock_type, sock_proto);
    if (((*ctx)->sock) < 0 )
        return NET_ERR;

    hst = gethostbyname(host);

    if (hst)
    {
        memcpy(&(((*ctx)->sa).sin_addr), hst->h_addr_list[0], hst->h_length);
    }
    else
    {
        (*ctx)->sa.sin_addr.s_addr = inet_addr(host);
    }

    (*ctx)->sa.sin_family = sock_fam;
    (*ctx)->sa.sin_port = htons(port);

#endif

    return 0;

}

void netDestroy(net_ctx **ctx)
{
    close((*ctx)->sock);
    free((*ctx));
}

int sndData(net_ctx **ctx, unsigned char data[20])
{

    return 0;
}

int getData(net_ctx **ctx, unsigned char data[20])
{

    return 0;
}

int connNet(net_ctx **ctx)
{

#if OS == 0x02
    if (connect((*ctx)->sock, (SOCKADDR*) &((*ctx)->sa), sizeof(((*ctx)->sa)))
            == SOCKET_ERROR)
    {
        close((*ctx)->sock);
        WSACleanup();
        return NET_ERR;
    }
#endif

#if OS == 0x03

    if (connect((*ctx)->sock, (struct sockaddr*) &((*ctx)->sa), sizeof(((*ctx)->sa))) < 0)
    {
        close((*ctx)->sock);
        return NET_ERR;
    }
#endif

    return 0;
}

int lisNet(net_ctx **ctx)
{

    return 0;
}

