/*
    $Id: sha.c 292 2010-02-14 07:37:34Z irfangul82 $
    Simple file and directory encryption
    Copyright (C) 2010  Irfan Gulamali

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include "sha.h"
#include "glob.h"

/* always call initSHA before this method
 * to initialize the SHA context
 * gets the next hash value based on the
 * data passed in.
 */
void nxtHash(shactx *ctx, char data[64])
{
    int modSize = 0;
    char *tdata = 0;
    char *tdata_head = 0;
    unsigned long long int sizeInBits = 0;
    unsigned short wCtr = 0;
    unsigned short mainCtr = 0;
    unsigned int W[80];
    unsigned int K = 0x0;
    unsigned int temp = 0x0;
    unsigned int a, b, c, d, e, f;
    short twoBlockFlag = 0;

    //hash already completed
    if (ctx->r_len == ctx->size)
        return;

    //determine if last block is being read
    if (ctx->r_len + 64 < ctx->size)
    {
        ctx->r_len += 64;
    }
    else if ((ctx->r_len + 64) > ctx->size)
    {
        modSize = ((ctx->size) % 64);
        ctx->r_len += modSize;

    }
    else if ((ctx->r_len + 64) == ctx->size)
    {
        modSize = -1;
        ctx->r_len += 64;
    }

    /*
     *file larger than 64bit length when
     *multiplied by 8 bits to get length in bits
     */

    if (ctx->size > 0xFFFFFFFFFFFFFFFULL )
    {
        ctx->size = (0xFFFFFFFFFFFFFFFULL);
    }

    sizeInBits = (unsigned long long) ((ctx->size) * 8);

    //preprocess now
    //last data block being read
    if (modSize)
    {
        (modSize < 56 && modSize != -1) ? (tdata = calloc(sizeof(char), 64))
        : (tdata = calloc(sizeof(char), 128));

        (modSize == -1) ? memcpy(tdata, data, 64)
        : memcpy(tdata, data, modSize);

        //append 1 to message block
        (modSize == -1) ? (tdata[64] ^= 0x80) : (tdata[modSize] ^= 0x80);
        //flag if multi block for last block
        ((modSize == -1) || (modSize >= 56)) ? (twoBlockFlag = 1)
        : (twoBlockFlag = 0);

    }
    else
    {
        tdata = calloc(sizeof(char), 64);
        memcpy(tdata, data, 64);
    }

    tdata_head = &(tdata[0]);

CHK_AGAIN:

    a = ctx->hash.h0;
    b = ctx->hash.h1;
    c = ctx->hash.h2;
    d = ctx->hash.h3;
    e = ctx->hash.h4;

    memset(&W, 0, sizeof(int) * 80);
    while (wCtr < 15)
    {
        wCtr == 0 ? memcpy(&W[wCtr], &tdata[wCtr], sizeof(int)) : memcpy(
            &W[wCtr], &tdata[(wCtr * 4)], sizeof(int));

        if ((wCtr != 14))
        {
            if (L_ENDIAN())
                REVBYTE32(W[wCtr]);
        }
        else
        {
            //if its the last block append the size in bits
            if (modSize && !twoBlockFlag)
            {
                if (L_ENDIAN())
                {
                    memcpy(&W[wCtr], &sizeInBits, 8);
                    memcpy(&W[wCtr+2],&W[wCtr+1],4); //swap
                    memcpy(&W[wCtr+1],&W[wCtr],4);
                    memcpy(&W[wCtr],&W[wCtr+2],4);
                    memset(&W[wCtr+2], 0, 4);


                }
                else
                {
                    memcpy(&W[wCtr], &sizeInBits, 8);
                }
            }
            else
            {
                memcpy(&W[wCtr], &tdata[(wCtr * 4)], sizeof(int));
                memcpy(&W[wCtr + 1], &tdata[((wCtr + 1) * 4)], sizeof(int));
                if (L_ENDIAN())
                {
                    REVBYTE32(W[wCtr]);
                    REVBYTE32(W[wCtr+1]);
                }
            }
        }
        wCtr++;
    }

    wCtr++;

    while (wCtr < 80)
    {
        W[wCtr] = LRT((W[wCtr-3] ^ W[wCtr-8] ^ W[wCtr-14] ^ W[wCtr-16]),1);
        wCtr++;
    }

    //main loop
    while (mainCtr < 80)
    {
        if (mainCtr <= 19)
        {
            K = 0x5A827999;
            f = (b & c) | ((~b) & d);

        }
        else if (20 <= mainCtr && mainCtr <= 39)
        {
            K = 0x6ED9EBA1;
            f = ((b) ^ (c)) ^ (d);

        }
        else if (40 <= mainCtr && mainCtr <= 59)
        {
            K = 0x8F1BBCDC;
            f = ((b & c) | (b & d)) | (c & d);

        }
        else if (60 <= mainCtr && mainCtr <= 79)
        {
            K = 0xCA62C1D6;
            f = ((b) ^ (c)) ^ (d);
        }
        temp = (LRT(a,5) + f + e + K + W[mainCtr]);
        e = d;
        d = c;
        c = LRT(b, 30);
        b = a;
        a = temp;

        mainCtr++;
    }

    //main loop ends---------------
    ctx->hash.h0 += a;
    ctx->hash.h1 += b;
    ctx->hash.h2 += c;
    ctx->hash.h3 += d;
    ctx->hash.h4 += e;

    if (twoBlockFlag)
    {
        twoBlockFlag = 0;
        wCtr = 0;
        mainCtr = 0;
        tdata = &tdata[64];
        goto CHK_AGAIN;
    }

    free(tdata_head);

}

/*
 * initializes the SHA context
 */
int initSHA(shactx **ctx, unsigned long long int size)
{
    //size is 0
    if (!size)
        return SHA_ERR;

    (*ctx) = malloc(sizeof(shactx));
    if ((*ctx) == NULL)
        return SHA_ERR;

    (*ctx)->hash.h0 = h0;
    (*ctx)->hash.h1 = h1;
    (*ctx)->hash.h2 = h2;
    (*ctx)->hash.h3 = h3;
    (*ctx)->hash.h4 = h4;
    (*ctx)->r_len = 0;

    (*ctx)->size = size;

    return 0;
}

/*
 * destroys the SHA context
 */
void destroySHA(shactx *ctx)
{
    free(ctx);
}

