/*
    $Id: util.c 297 2010-02-16 21:38:42Z irfangul82 $
    Simple file and directory encryption
    Copyright (C) 2010  Irfan Gulamali

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <math.h>
#include <time.h>
#include <limits.h>

#if OS == 0x02
#include <windows.h>
#endif

#include "util.h"
#include "sha.h"
#include "aes.h"
#include "glob.h"
#include "thread.h"
#include "mutex.h"
#include "rand.h"
#include "base64.h"

#if OS == 0x02 //win
HANDLE *mtx1;
HANDLE *mtx2;
HANDLE *mtx3;
HANDLE *mtx4;
#endif

#if OS == 0x03 //linux
pthread_mutex_t *mtx1;
pthread_mutex_t *mtx2;
pthread_mutex_t *mtx3;
pthread_mutex_t *mtx4;
#endif

#define READBLKSIZE_T 8388608
#define MAX_MSG_SIZE 524288
#define MAX_MSG_DEC_SIZE 1048576

/* @pars_fl should be freed
 * by the calling function
 */
void parseFileName(unsigned char *fname, unsigned char **pars_fl)
{
    int slashCnt = 0;
    int filepathLen = 0;
    int cntr_0 = 0;
    int cntr_1 = 0;

    while (fname[filepathLen] != '\0')
    {
        if (fname[filepathLen] == '\\')
            slashCnt++;
        filepathLen++;
    }
    (*pars_fl) = malloc(sizeof(char) * (strlen(fname)) + slashCnt + 1);
    memset(*pars_fl, 0, sizeof(char) * (strlen(fname)) + slashCnt + 1);

    while (cntr_0 < filepathLen)
    {
        if (fname[cntr_0] == '\\')
        {
            (*pars_fl)[cntr_1] = '\\';
            cntr_1++;
            (*pars_fl)[cntr_1] = '\\';
            cntr_1++;
            cntr_0++;
        }
        else
        {
            (*pars_fl)[cntr_1] = fname[cntr_0];
            cntr_0++;
            cntr_1++;
        }
    }
}

int encryptFile(unsigned char *fname, unsigned char* encKey)
{
    FILE *fp_r, *fp_w;
    int status;
    struct stat st;
    aesctx *aes_ctx;
    unsigned char fname_o[255];
    unsigned char hash[41] = "";
    unsigned char enc_sufx[] = ".enc";
    unsigned long long file_sz = 0;
    unsigned long long bytes_read = 0;
    unsigned long long size_fread = 0;
    unsigned char *readBlk;
    unsigned int procBlkCnt = 0;
    unsigned char TMPBUFF1[16] = "";
    unsigned char TMPBUFF2[16] = "";
    int cntr = 0;

    readBlk = (char *) malloc(READBLKSIZE_T);
    //first determine the hash of the file
    status = gtHash(fname, hash);
    if (status)
        return status;

    fp_r = fopen(fname, "rb");
    if (fp_r == NULL)
    {
        free(readBlk);
        return FILEIO_ERROR;
    }

    //determine size of file to encrypt
    if (stat(fname, &st))
    {
        free(readBlk);
        return FILEIO_ERROR;
    }
    file_sz = st.st_size;

    //open a file to output the encrypted file
    memset(fname_o, 0, 255);
    strncpy(fname_o, fname, strlen(fname));
    if ((strstr(fname, enc_sufx)) == NULL)
    {
        strncat(fname_o, enc_sufx, strlen(enc_sufx));
    }
    else
    {
        fclose(fp_r);
        return ENC_EXT;
    }

    fp_w = fopen(fname_o, "wb");
    if (fp_w == NULL)
    {
        fclose(fp_r);
        free(readBlk);
        return FILEIO_ERROR;
    }

    //create aes context
    initAESCTX(&aes_ctx, encKey);

    //create random iv
    k_rand(&TMPBUFF1[0], 16);
    aes_ctx->BLK = &TMPBUFF1[0];
    fwrite(aes_ctx->BLK, 1, 16, fp_w);
    encryptAES256(aes_ctx);

    //append hash to file
    cntr = 0;
    while ( cntr < 16 )
    {
        TMPBUFF1[cntr] ^= hash[cntr];
        cntr++;
    }
    fwrite(aes_ctx->BLK, 1, 16, fp_w);
    encryptAES256(aes_ctx);

    cntr = 0;
    while ( cntr < 16 )
    {
        TMPBUFF1[cntr] ^= hash[16+cntr];
        cntr++;
    }
    fwrite(aes_ctx->BLK, 1, 16, fp_w);
    encryptAES256(aes_ctx);

    memset(TMPBUFF2, 0, 16);
    memcpy(TMPBUFF2, &hash[32], 8);
    cntr = 0;
    while ( cntr < 16 )
    {
        TMPBUFF1[cntr] ^= TMPBUFF2[cntr];
        cntr++;
    }
    fwrite(aes_ctx->BLK, 1, 16, fp_w);
    encryptAES256(aes_ctx);

    //append file size to file header
    memset(TMPBUFF2, 0, 16);
    memcpy(TMPBUFF2, &file_sz, sizeof(file_sz));
    cntr = 0;
    while ( cntr < 16 )
    {
        TMPBUFF1[cntr] ^= TMPBUFF2[cntr];
        cntr++;
    }
    fwrite(aes_ctx->BLK, 1, 16, fp_w);
    encryptAES256(aes_ctx);

    //process file encrypting it
    while ((bytes_read < file_sz) && (!feof(fp_r)))
    {
        memset(readBlk, 0, READBLKSIZE_T);
        size_fread = fread(readBlk, 1, READBLKSIZE_T, fp_r);
        bytes_read += size_fread;

        while (procBlkCnt < ceil(size_fread/16.0))
        {
            cntr = 0;
            while ( cntr < 16 )
            {
                TMPBUFF1[cntr] ^= readBlk[procBlkCnt*16+cntr];
                cntr++;
            }
            memcpy(&readBlk[procBlkCnt*16], TMPBUFF1, 16);
            encryptAES256(aes_ctx);
            procBlkCnt++;
        }
        procBlkCnt = 0;
        fwrite(readBlk, 1, (16*ceil(size_fread/16.0)), fp_w);
        fflush(fp_w);

    }

    fclose(fp_w);
    fclose(fp_r);
    free(readBlk);
    remove(fname); //remove the file
    destroyAESCTX(&aes_ctx);
    return 0;
}

int decryptFile(unsigned char *fname, unsigned char* encKey)
{
    FILE *fp_r, *fp_w;
    struct stat st;
    aesctx *aes_ctx;
    unsigned char fname_o[255];
    unsigned char enc_sufx[] = ".enc";
    unsigned char dec_sufx[] = ".dec";
    unsigned char hash[41] = ""; //store hash from decryption
    unsigned char hashVerify[41] = "";
    unsigned long long file_sz = 0;
    unsigned long long bytes_read = 0;
    unsigned long long size_fread = 0;
    int bytes_otstnding = 0;
    unsigned char *readBlk;
    unsigned int procBlkCnt = 0;
    unsigned char TMPBUFF1[16] = "";
    unsigned char TMPBUFF2[16] = "";
    int cntr = 0;

    readBlk = (char *) malloc(READBLKSIZE_T);

    memset(fname_o, 0, (strlen(fname) + 4));
    fp_r = fopen(fname, "rb");

    if (fp_r == NULL)
    {
        free(readBlk);
        return FILEIO_ERROR;
    }

    if ((strstr(fname, enc_sufx)) != NULL)
    {
        strncpy(fname_o, fname, strlen(fname) - (strlen(enc_sufx)));
    }
    else
    {
        fclose(fp_r);
        return ENC_EXT;
    }

    fp_w = fopen(fname_o, "wb");
    if (fp_w == NULL)
    {
        fclose(fp_r);
        free(readBlk);
        return FILEIO_ERROR;
    }

    initAESCTX(&aes_ctx, encKey);

    //read random iv and get hash from file
    fread(TMPBUFF1, 1, 16, fp_r);
    aes_ctx->BLK = &TMPBUFF1[0];
    encryptAES256(aes_ctx);
    fread(TMPBUFF2, 1, 16, fp_r);
    cntr = 0;
    while ( cntr < 16 )
    {
        TMPBUFF1[cntr] ^= TMPBUFF2[cntr];
        cntr++;
    }
    memcpy(&hash[0], &TMPBUFF1[0], 8);
    memcpy(&hash[8], &TMPBUFF1[8], 8);
    aes_ctx->BLK = &TMPBUFF2[0];
    encryptAES256(aes_ctx);
    fread(TMPBUFF1, 1, 16, fp_r);
    cntr = 0;
    while ( cntr < 16 )
    {
        TMPBUFF2[cntr] ^= TMPBUFF1[cntr];
        cntr++;
    }
    memcpy(&hash[16], &TMPBUFF2[0], 8);
    memcpy(&hash[24], &TMPBUFF2[8], 8);
    aes_ctx->BLK = &TMPBUFF1[0];
    encryptAES256(aes_ctx);
    fread(TMPBUFF2, 1, 16, fp_r);
    cntr = 0;
    while ( cntr < 16 )
    {
        TMPBUFF1[cntr] ^= TMPBUFF2[cntr];
        cntr++;
    }
    memcpy(&hash[32], &TMPBUFF1[0], 8);
    aes_ctx->BLK = &TMPBUFF2[0];
    encryptAES256(aes_ctx);

    //get original size of file from the header
    fread(TMPBUFF1, 1, 16, fp_r);
    cntr = 0;
    while ( cntr < 16 )
    {
        TMPBUFF2[cntr] ^= TMPBUFF1[cntr];
        cntr++;
    }
    memcpy(&file_sz, aes_ctx->BLK, sizeof(file_sz));
    aes_ctx->BLK = &TMPBUFF1[0];
    encryptAES256(aes_ctx);

    while ((bytes_read < file_sz) && (!feof(fp_r)))
    {
        memset(readBlk, 0, READBLKSIZE_T);
        size_fread = fread(readBlk, 1, READBLKSIZE_T, fp_r);
        while (procBlkCnt < ceil(size_fread/16.0))
        {
            cntr = 0;
            while ( cntr < 16 )
            {
                TMPBUFF1[cntr] ^= readBlk[procBlkCnt*16+cntr];
                cntr++;
            }
            memcpy(TMPBUFF2,&readBlk[procBlkCnt*16], 16);
            memcpy(&readBlk[procBlkCnt*16], TMPBUFF1, 16);
            memcpy(TMPBUFF1, TMPBUFF2, 16);
            encryptAES256(aes_ctx);
            procBlkCnt++;
        }
        procBlkCnt = 0;

        if (bytes_read + size_fread < file_sz)
        {
            bytes_read += size_fread;
            fwrite(readBlk, 1, (16*ceil(size_fread/16.0)), fp_w);
            fflush(fp_w);
        }
        else
        {
            bytes_otstnding = file_sz - bytes_read;
            bytes_read += bytes_otstnding;
            fwrite(readBlk, 1, bytes_otstnding, fp_w);
            fflush(fp_w);
        }
    }

    fclose(fp_w);
    fclose(fp_r);
    free(readBlk);
    destroyAESCTX(&aes_ctx);

    //verify hash
    gtHash(fname_o, hashVerify);

    if ((strncmp(hash, hashVerify, 40)))
    {
        remove(fname_o); //remove the file
        return SHA_ERR;
    }

    remove(fname); //remove the file
    return 0;
}

int encryptDir(unsigned char *fname, unsigned char* encKey)
{
    unsigned char cache_fname[PATH_MAX];
    unsigned char pars_cache_fname[PATH_MAX];
    struct stat st;
    short int cntr = 0;
    unsigned char hash[41] = "";
    FILE *fpErrCache = 0x0;
    FILE *fpCache = 0x0;
    int ERR_PRESENT = 0;
    unsigned char *parse_fname= 0x0;

    fpErrCache = fopen(PARS_ERR_CACHE, "a+");
    if (!fpErrCache)
    {
        return FILEIO_ERROR;
    }

    if (stat(PARS_ERR_CACHE, &st))
        return FILEIO_ERROR;

    if (st.st_size)
    {
        return -1;
    }
    parseDirDBClean();
    if (parseDirTree(fname))
    {
        fclose(fpErrCache);
        return FILEIO_ERROR;
    }

    fpCache = fopen(PARS_DIR_PATH,"r");
    if (!fpCache)
    {
        fclose(fpErrCache);
        return FILEIO_ERROR;
    }

    while (!feof(fpCache))
    {
        memset(cache_fname, 0, PATH_MAX);
        fgets(cache_fname, PATH_MAX, fpCache);
        cache_fname[strlen(cache_fname)-1]='\0';
        if (strlen(cache_fname))
        {
            if (encryptFile(cache_fname, encKey))
            {
                if (!ERR_PRESENT)
                    ERR_PRESENT = 1;
                fflush(fpErrCache);
            }
        }
    }

    if (ERR_PRESENT)
    {
        fclose(fpCache);
        fclose(fpErrCache);
        return 0;
    }
    else
    {
        fclose(fpCache);
        fclose(fpErrCache);
        remove(PARS_ERR_CACHE);
    }
    parseDirDBClean();
    return 0;
}

int decryptDir(unsigned char *fname, unsigned char* encKey)
{
    unsigned char cache_fname[PATH_MAX];
    unsigned char pars_cache_fname[PATH_MAX];
    struct stat st;
    short int cntr = 0;
    unsigned char hash[41] = "";
    FILE *fpErrCache = 0x0;
    FILE *fpCache = 0x0;
    int ERR_PRESENT = 0;
    unsigned char *parse_fname= 0x0;

    fpErrCache = fopen(PARS_ERR_CACHE, "a+");
    if (!fpErrCache)
    {
        return FILEIO_ERROR;
    }

    if (stat(PARS_ERR_CACHE, &st))
        return FILEIO_ERROR;

    if (st.st_size)
    {
        return -1;
    }
    parseDirDBClean();
    if (parseDirTree(fname))
    {
        fclose(fpErrCache);
        return FILEIO_ERROR;
    }

    fpCache = fopen(PARS_DIR_PATH,"r");
    if (!fpCache)
    {
        fclose(fpErrCache);
        return FILEIO_ERROR;
    }

    while (!feof(fpCache))
    {
        memset(cache_fname, 0, PATH_MAX);
        fgets(cache_fname, PATH_MAX, fpCache);
        cache_fname[strlen(cache_fname)-1]='\0';
        if (strlen(cache_fname))
        {
            if (decryptFile(cache_fname, encKey))
            {
                if (!ERR_PRESENT)
                    ERR_PRESENT = 1;
                fflush(fpErrCache);
            }
        }
    }

    if (ERR_PRESENT)
    {
        fclose(fpCache);
        fclose(fpErrCache);
        return 0;
    }
    else
    {
        fclose(fpCache);
        fclose(fpErrCache);
        remove(PARS_ERR_CACHE);
    }
    parseDirDBClean();
    return 0;
}


int gtHash(unsigned char *fname, unsigned char *hash)
{
    FILE *fp_r;
    shactx *shaTX;
    unsigned char data[64];
    struct stat st;
    unsigned long long int file_sz = 0;
    unsigned long long int readBytes = 0;

    if (stat(fname, &st))
        return FILEIO_ERROR;
    file_sz = st.st_size;
    if (!file_sz) //zero files hashes aren't done
        return 0;
    fp_r = fopen(fname, "rb");
    if (fp_r == NULL)
        return FILEIO_ERROR;

    initSHA(&shaTX, file_sz);

    while ((readBytes < file_sz) && (!feof(fp_r)))
    {
        readBytes += fread(data, 1, 64, fp_r);
        nxtHash(shaTX, data);
    }

    sprintf(&hash[0], "%08x", shaTX->hash.h0);
    sprintf(&hash[8], "%08x", shaTX->hash.h1);
    sprintf(&hash[16], "%08x", shaTX->hash.h2);
    sprintf(&hash[24], "%08x", shaTX->hash.h3);
    sprintf(&hash[32], "%08x", shaTX->hash.h4);

    destroySHA(shaTX);

    fclose(fp_r);
    return 0;
}

int gtHashDir(unsigned char *fname)
{
    unsigned char cache_fname[PATH_MAX];
    unsigned char pars_cache_fname[PATH_MAX];
    struct stat st;
    short int cntr = 0;
    unsigned char hash[41] = "";
    FILE *fpErrCache = 0x0;
    FILE *fpCache = 0x0;
    int ERR_PRESENT = 0;
    unsigned char *parse_fname= 0x0;

    fpErrCache = fopen(PARS_ERR_CACHE, "a+");
    if (!fpErrCache)
    {
        return FILEIO_ERROR;
    }

    if (stat(PARS_ERR_CACHE, &st))
        return FILEIO_ERROR;

    if (st.st_size)
    {
        return -1;
    }
    parseDirDBClean();
    if (parseDirTree(fname))
    {
        fclose(fpErrCache);
        return FILEIO_ERROR;
    }

    fpCache = fopen(PARS_DIR_PATH,"r");
    if (!fpCache)
    {
        fclose(fpErrCache);
        return FILEIO_ERROR;
    }

    while (!feof(fpCache))
    {
        memset(cache_fname, 0, PATH_MAX);
        fgets(cache_fname, PATH_MAX, fpCache);
        cache_fname[strlen(cache_fname)-1]='\0';
        if (strlen(cache_fname))
        {
            if (gtHash(cache_fname, hash))
            {
                if (!ERR_PRESENT)
                    ERR_PRESENT = 1;
                fflush(fpErrCache);
            }
            else
            {
                while (cntr < 40)
                {
                    cntr++;
                }
                cntr = 0;
            }
        }
    }

    if (ERR_PRESENT)
    {
        fclose(fpCache);
        fclose(fpErrCache);
        return 0;
    }
    else
    {
        fclose(fpCache);
        fclose(fpErrCache);
        remove(PARS_ERR_CACHE);
    }
    parseDirDBClean();
    return 0;
}


/*
  creates a local disk cache of the directory
  tree that is passed in
*/

/****** LINUX OS */
#if OS == 0x03
int parseDirTree(unsigned char *path)
{
    DIR *dip;
    struct dirent *dit;
    unsigned char *fpath;
    unsigned char TMP_PATH[PATH_MAX];

    FILE *fp = NULL;

    parseFileName(path, &fpath);

    if ( (strcmp(fpath,".") !=0) && (strcmp(fpath,"..")!=0))
        dip = opendir(fpath);

    if (dip == NULL)
    {
        return -1;
    }

    while ( (dit = readdir(dip))!= NULL )
    {
        //open cache file
        if (fp == NULL)
        {
            fp = fopen(PARS_DIR_PATH, "a");
            if ( fp == NULL)
            {
                fclose(fp);
                return -1;
            }
        }


        //if this is a file do something
        if ( dit->d_type == DT_REG)
        {
            if (strlen(path)+1+strlen(dit->d_name)< PATH_MAX)
            {
                memset(TMP_PATH, 0, PATH_MAX);
                strncat(TMP_PATH, path, strlen(path));
                strncat(TMP_PATH,"/", strlen("/"));
                strncat(TMP_PATH,dit->d_name, strlen(dit->d_name));

                fprintf(fp, "%s\n", TMP_PATH);
                fflush(fp);
            }
        }

        //if this is a directory do something
        if ( dit->d_type == DT_DIR && (strcmp(dit->d_name,".") !=0) && \
                (strcmp(dit->d_name,"..")!=0))
        {
            if (strlen(path)+1+strlen(dit->d_name)< PATH_MAX)
            {
                //close cache file as we may need
                //to open the cache file recursively in a call
                fclose(fp);
                fp = NULL;
                memset(TMP_PATH, 0, PATH_MAX);
                strncat(TMP_PATH, path, strlen(path));
                strncat(TMP_PATH,"/", strlen("/"));
                strncat(TMP_PATH,dit->d_name, strlen(dit->d_name));
                parseDirTree(TMP_PATH);
            }
        }
    }

    closedir(dip);
    free(fpath);
    if (fp != NULL)
        fclose(fp);

    return 0;
}
#endif

/****** WIN OS */
#if OS == 0x02
int parseDirTree(unsigned char *path)
{
    WIN32_FIND_DATA fdata;
    HANDLE hfind;
    unsigned char srch[] = "\\*";
    unsigned char *fpath;
    unsigned char SRCH_PATH[PATH_MAX];
    unsigned char TMP_PATH[PATH_MAX];
    FILE *fp = NULL;
    int ret = 0;

    parseFileName(path, &fpath);

    memset(SRCH_PATH, 0, PATH_MAX);
    strcat(SRCH_PATH, fpath);
    strcat(SRCH_PATH, srch);

    hfind = FindFirstFile( SRCH_PATH, &fdata);
    if (hfind == INVALID_HANDLE_VALUE )
        return -1;

    do
    {
        //open cache file
        if (fp == NULL)
        {
            fp = fopen(PARS_DIR_PATH, "a");
            if ( fp == NULL)
            {
                fclose(fp);
                return -1;
            }
        }
        if ((strcmp(fdata.cFileName,".") !=0) && (strcmp(fdata.cFileName,"..")!=0))
        {
            if (fdata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
            {
                //close cache file as we may need
                //to open the cache file recursively in a call
                fclose(fp);
                fp = NULL;
                memset(TMP_PATH, 0, PATH_MAX);
                strncat(TMP_PATH,path, strlen(path));
                strncat(TMP_PATH,"\\", strlen("\\"));
                strncat(TMP_PATH,fdata.cFileName, strlen(fdata.cFileName));
                parseDirTree(TMP_PATH);
            }
            else
            {
                if (strlen(path)+1+strlen(fdata.cFileName)< PATH_MAX)
                {
                    memset(TMP_PATH, 0, PATH_MAX);
                    strncat(TMP_PATH,path, strlen(path));
                    strncat(TMP_PATH,"\\", strlen("\\"));
                    strncat(TMP_PATH,fdata.cFileName, strlen(fdata.cFileName));
					fprintf(fp, "%s\n", TMP_PATH);
                    fflush(fp);
                }
            }
        }
        ret = FindNextFile(hfind, &fdata);
    }
    while ( ret != 0 && ret != 18);

    free(fpath);
    if (fp != NULL)
        fclose(fp);
    return 0;
}
#endif


/* Clears parsedir cache file */
int parseDirDBClean()
{
    FILE *fp;
    fp = fopen(PARS_DIR_PATH, "w");
    if ( fp == NULL)
    {
        return -1;
    }
    fclose(fp);
    remove(PARS_DIR_PATH);
    return 0;
}

int encMsg(unsigned char *fname, unsigned char* encKey)
{
    int status = 0;
    int file = 0;
    struct stat st;
    unsigned long long int file_sz = 0;
    unsigned char *file_buff = 0;
    unsigned char *base64_buff = 0;
    unsigned char fname_parsed[PATH_MAX] = "";
    FILE *fp_r, *fp_w;
    int cntr = 0;
    int base64_sz = 0;
    int column_offset = 0;
    const int colsz = 60;

    if (stat(fname, &st))
        return FILEIO_ERROR;
    file_sz = st.st_size;
    if (!file_sz)
        return 0;

    if ( file_sz > MAX_MSG_SIZE)
        return FILESZ_OUTBOUND;

    strcpy(fname_parsed, fname);
    strcat(fname_parsed, ".enc");

    status = encryptFile(fname, encKey);
	
    if (status)
    return status;

    if (stat(fname_parsed, &st))
        return FILEIO_ERROR;
    file_sz = st.st_size;
    if (!file_sz)
        return 0;

    if ( file_sz > MAX_MSG_SIZE)
        return FILESZ_OUTBOUND;

    fp_r = fopen(fname_parsed, "rb");
    if (fp_r == NULL)
        return FILEIO_ERROR;

    fp_w = fopen(fname, "w");
    if (fp_w == NULL)
    {
        fclose(fp_r);
        return FILEIO_ERROR;
    }
    file_buff = malloc(file_sz);

    fread(file_buff, 1, file_sz, fp_r);
    fclose(fp_r);

    base64_sz = base64Enc(file_buff, &base64_buff, file_sz);
    column_offset = base64_sz % 60;
    cntr = 0;
   
    while (cntr < base64_sz)
    {
        if ( base64_sz < 60 )
        {
            fwrite(&base64_buff[0], 1, base64_sz, fp_w);
            cntr = base64_sz;
        }
        if ( cntr != base64_sz)
        {
            if ( (cntr+60) < base64_sz)
            {
                fwrite(&base64_buff[cntr], 1, 60, fp_w);
                fprintf(fp_w, "\n");
                cntr+=60;
            }
            else
            {
                fwrite(&base64_buff[cntr], 1, column_offset, fp_w);
                fprintf(fp_w, "\n");
                cntr = base64_sz;
            }
        }
    }
    fclose(fp_w);
    remove(fname_parsed);
    free(file_buff);
    free(base64_buff);
    return 0;
}

int decMsg(unsigned char *fname, unsigned char* encKey)
{
    const int noasci = 123;
    int status = 0;
    int file = 0;
    struct stat st;
    unsigned long long int file_sz = 0;
    unsigned char *file_buff = 0;
    unsigned char *base64_buff = 0;
    unsigned char *base64_buff_tmp = 0;
    unsigned char fname_parsed[PATH_MAX] = "";
    unsigned char fname_parsed2[PATH_MAX] = "";
    FILE *fp_r, *fp_w;
    int cntr = 0;
    int cntr2 = 0;
    int base64_sz = 0;
    int column_offset = 0;
    const int colsz = 60;

    strcpy(fname_parsed, fname);
    strcat(fname_parsed, ".tmp.enc");
    strcpy(fname_parsed2, fname);
    strcat(fname_parsed2, ".tmp");

    if (stat(fname, &st))
        return FILEIO_ERROR;
    file_sz = st.st_size;
    if (!file_sz)
        return 0;

    if ( file_sz > MAX_MSG_DEC_SIZE)
        return FILESZ_OUTBOUND;

    fp_r = fopen(fname, "r");
    if (fp_r == NULL)
        return FILEIO_ERROR;

    fp_w = fopen(fname_parsed, "wb");
    if (fp_w == NULL)
    {
        fclose(fp_r);
        return FILEIO_ERROR;
    }
    base64_buff = calloc(file_sz+1, 1);
    memset(base64_buff, 0, file_sz+1);

    base64_buff_tmp = calloc(file_sz+1, 1);
    fread(base64_buff_tmp, 1, file_sz, fp_r);
    fclose(fp_r);

    cntr = 0;
    while ( cntr < file_sz)
    {

        if ( base64_buff_tmp[cntr] < noasci )
        {
            cntr++;
        }
        else
        {
	    free(base64_buff);
	    free(base64_buff_tmp);
            return FILEFORMAT_ERROR;
        }
    }

    cntr = 0;
    cntr2 = 0;
    while ( cntr < file_sz)
    {
        if ( base64_buff_tmp[cntr] != '\n' && base64_buff_tmp[cntr] != '\0')
        {
            base64_buff[cntr2] = base64_buff_tmp[cntr];
            cntr2++;
        }
        cntr++;
    }

    base64_sz = base64Dec(&file_buff, base64_buff, cntr2);
    fwrite(file_buff, 1, base64_sz, fp_w);
    fclose(fp_w);

    status = decryptFile(fname_parsed, encKey);
	
    free(file_buff);
    free(base64_buff);
    free(base64_buff_tmp);


    if (status)
        return status;

    remove(fname);
    rename(fname_parsed2, fname);
    return 0;
}
