/*
    $Id: directorypanel.cpp 301 2010-02-17 21:00:48Z irfangul82 $
    Simple file and directory encryption
    Copyright (C) 2010  Irfan Gulamali

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "directorypanel.h"
#include "kcore.h"

directorypanel::directorypanel(QWidget* parent) : QWidget(parent)
{
    directoryLabel = new QLabel(tr("directory: "), this);
    passwordLabel = new QLabel(tr("password: "), this);
    pathEdit = new QLineEdit(this);
    messageBox = new QTextEdit(this);
    browseButton = new QPushButton(tr("&Browse"), this);
    encryptButton = new QPushButton(tr("&Encrypt"), this);
    decryptButton = new QPushButton(tr("De&crypt"), this);
    password1 = new QLineEdit(this);
    password2 = new QLineEdit(this);
    v1 = new QVBoxLayout(this);
    h1 = new QHBoxLayout(this);
    h2 = new QHBoxLayout(this);
    h3 = new QHBoxLayout(this);
    h4 = new QHBoxLayout(this);

    messageBox->setReadOnly(true);

    encryptButton->setDisabled(true);
    decryptButton->setDisabled(true);
    password1->setEchoMode(QLineEdit::Password);
    password2->setEchoMode(QLineEdit::Password);

    v1->addItem(h1);
    v1->addItem(h2);
    v1->addItem(h3);
    v1->addItem(h4);

    h1->addWidget(directoryLabel);
    h1->addWidget(pathEdit);
    h1->addWidget(browseButton);

    h2->addWidget(passwordLabel);
    h2->addWidget(password1);
    h2->addWidget(password2);

    h3->addWidget(encryptButton);
    h3->addWidget(decryptButton);

    h4->addWidget(messageBox);

    password1->setMaxLength(32);
    password2->setMaxLength(32);

    setLayout(v1);

    connect(browseButton, SIGNAL(clicked()), SLOT(getDirectoryFromBrowser()));
    connect(pathEdit, SIGNAL(textChanged (const QString&)), this, SLOT(validateForm(const QString&)));
    connect(password1, SIGNAL(textEdited (const QString&)), this, SLOT(validateForm(const QString&)));
    connect(password2, SIGNAL(textEdited (const QString&)), this, SLOT(validateForm(const QString&)));
    connect(encryptButton, SIGNAL(clicked()), SLOT(encryptDirectory()));
    connect(decryptButton, SIGNAL(clicked()), SLOT(decryptDirectory()));

}


directorypanel::~directorypanel()
{
}

void directorypanel::getDirectoryFromBrowser()
{
    pathEdit->setText(QFileDialog::getExistingDirectory(this,tr("Find directory"),"/"));
}

void directorypanel::validateForm(const QString&)
{
    int toggleButton = 0;
    int togglePath = 0;

    if (((pathEdit->text()).compare(tr(""))))
        togglePath = 1;

    if (!((password1->text()).compare(password2->text())) &&
            (password1->text().compare(tr(""))) && (password2->text().compare(tr(""))))
        toggleButton = 1;

    if ( (toggleButton*togglePath) )
    {
        encryptButton->setDisabled(false);
        decryptButton->setDisabled(false);
    }
    else
    {
        encryptButton->setDisabled(true);
        decryptButton->setDisabled(true);
    }
}


void directorypanel::decryptDirectory()
{
    QWidget::setCursor(QCursor(Qt::WaitCursor)) ;
    browseButton->setDisabled(true);
    encryptButton->setDisabled(true);
    decryptButton->setDisabled(true);

    messageBox->clear();
    messageBox->append("Parsing dir...");
    messageBox->parentWidget()->repaint();
    kcore::decryptKDir(pathEdit->text().toStdString(), password1->text().toStdString(), messageBox);

    browseButton->setDisabled(false);
    encryptButton->setDisabled(false);
    decryptButton->setDisabled(false);
    QWidget::setCursor(QCursor(Qt::PointingHandCursor));
}


void directorypanel::encryptDirectory()
{
    QWidget::setCursor(QCursor(Qt::WaitCursor));
    browseButton->setDisabled(true);
    encryptButton->setDisabled(true);
    decryptButton->setDisabled(true);

    messageBox->clear();
    messageBox->append("Parsing dir...");
    messageBox->parentWidget()->repaint();
    kcore::encryptKDir(pathEdit->text().toStdString(), password1->text().toStdString(), messageBox);

    browseButton->setDisabled(false);
    encryptButton->setDisabled(false);
    decryptButton->setDisabled(false);
    QWidget::setCursor(QCursor(Qt::PointingHandCursor)) ;
}

#if OS == 0x03
#include "directorypanel.moc"
#endif