/*
    $Id: filepanel.cpp 301 2010-02-17 21:00:48Z irfangul82 $
    Simple file and directory encryption
    Copyright (C) 2010  Irfan Gulamali

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "filepanel.h"
#include "kcore.h"

#include <cstdlib>

filepanel::filepanel(QWidget* parent) : QWidget(parent)
{
    fileLabel = new QLabel(tr("file: "), this);
    passwordLabel = new QLabel(tr("password: "), this);
    filepathEdit = new QLineEdit(this);
    password1 = new QLineEdit(this);
    password2 = new QLineEdit(this);
    msgBox = new QTextEdit(this);
    v1 = new QVBoxLayout(this);
    h1 = new QHBoxLayout(this);
    h2 = new QHBoxLayout(this);
    h3 = new QHBoxLayout(this);
    h4 = new QHBoxLayout(this);
    browseButton = new QPushButton(tr("&Browse"), this);
    encryptButton = new QPushButton(tr("&Encrypt"), this);
    decryptButton = new QPushButton(tr("De&crypt"), this);

    encryptButton->setDisabled(true);
    decryptButton->setDisabled(true);

    password1->setMaxLength(32);
    password2->setMaxLength(32);

    password1->setEchoMode(QLineEdit::Password);
    password2->setEchoMode(QLineEdit::Password);

    msgBox->setReadOnly(true);

    v1->addItem(h1);
    v1->addItem(h2);
    v1->addItem(h3);
    v1->addItem(h4);

    v1->setAlignment(h1,Qt::AlignTop);
    v1->setAlignment(h2,Qt::AlignTop);
    v1->setAlignment(h3,Qt::AlignTop);

    h1->addWidget(fileLabel,0);
    h1->addWidget(filepathEdit,1);
    h1->addWidget(browseButton, 0);

    h2->addWidget(passwordLabel, 0);
    h2->addWidget(password1, 1);
    h2->addWidget(password2, 1);

    h3->addWidget(encryptButton);
    h3->addWidget(decryptButton);

    h4->addWidget(msgBox);

    setLayout(v1);

    connect(browseButton, SIGNAL(clicked()), SLOT(getFileFromBrowser()));
    connect(filepathEdit, SIGNAL(textChanged (const QString&)), this, SLOT(validateForm(const QString&)));
    connect(password1, SIGNAL(textEdited (const QString&)), this, SLOT(validateForm(const QString&)));
    connect(password2, SIGNAL(textEdited (const QString&)), this, SLOT(validateForm(const QString&)));
    connect(encryptButton, SIGNAL(clicked()), SLOT(encryptFile()));
    connect(decryptButton, SIGNAL(clicked()), SLOT(decryptFile()));

}


filepanel::~filepanel()
{
}


void filepanel::getFileFromBrowser()
{
    filepathEdit->setText(QFileDialog::getOpenFileName(this,tr("Find file"),"/","*", 0));
}


void filepanel::validateForm(const QString&)
{
    int toggleButton = 0;
    int togglePath = 0;

    if (((filepathEdit->text()).compare(tr(""))))
        togglePath = 1;

    if (!((password1->text()).compare(password2->text())) &&
            (password1->text().compare(tr(""))) && (password2->text().compare(tr(""))))
        toggleButton = 1;

    if ( (toggleButton*togglePath) )
    {
        encryptButton->setDisabled(false);
        decryptButton->setDisabled(false);
    }
    else
    {
        encryptButton->setDisabled(true);
        decryptButton->setDisabled(true);
    }
}

void filepanel::encryptFile()
{
    QString returnMessage();
    int unsigned ret = 0;
    QWidget::setCursor(QCursor(Qt::WaitCursor));
    cursor().setShape(Qt::BusyCursor);
    encryptButton->setDisabled(true);
    decryptButton->setDisabled(true);
    browseButton->setDisabled(true);

    msgBox->clear();
    msgBox->parentWidget()->repaint();
    ret = kcore::encryptKFile(filepathEdit->text().toStdString(), password1->text().toStdString());
    msgBox->append(tr(kcore::errorMessage(ret).c_str()));
    repaint();

    browseButton->setDisabled(false);
    encryptButton->setDisabled(false);
    decryptButton->setDisabled(false);
    QWidget::setCursor(QCursor(Qt::PointingHandCursor));
}

void filepanel::decryptFile()
{
    QString returnMessage();
    int unsigned ret = 0;
    QWidget::setCursor(QCursor(Qt::WaitCursor));
    encryptButton->setDisabled(true);
    decryptButton->setDisabled(true);
    browseButton->setDisabled(true);

    msgBox->clear();
    msgBox->parentWidget()->repaint();
    ret = kcore::decryptKFile(filepathEdit->text().toStdString(), password1->text().toStdString());
    msgBox->append(tr(kcore::errorMessage(ret).c_str()));

    browseButton->setDisabled(false);
    encryptButton->setDisabled(false);
    decryptButton->setDisabled(false);
    QWidget::setCursor(QCursor(Qt::PointingHandCursor));
}

#if OS == 0x03
#include "filepanel.moc"
#endif