/*
    $Id: hashpanel.cpp 301 2010-02-17 21:00:48Z irfangul82 $
    Simple file and directory encryption
    Copyright (C) 2010  Irfan Gulamali

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "hashpanel.h"
#include "kcore.h"

hashpanel::hashpanel(QWidget* parent) : QWidget(parent)
{
    v1 = new QVBoxLayout(this);
    h1 = new QHBoxLayout(this);
    h2 = new QHBoxLayout(this);
    h3 = new QHBoxLayout(this);
    h4 = new QHBoxLayout(this);

    v1->addItem(h1);
    v1->addItem(h2);
    v1->addItem(h3);

    dirpathEdit = new QLineEdit(this);
    filepathEdit = new QLineEdit(this);
    messageBox = new QTextEdit(this);
    directoryLabel = new QLabel(tr("directory: "), this);
    fileLabel = new QLabel(tr("file: "), this);
    gethashButton1 = new QPushButton(tr("Hash"),this);
    gethashButton2 = new QPushButton(tr("Hash"),this);
    browseButton1 = new QPushButton(tr("Br&owse"), this);
    browseButton2 = new QPushButton(tr("Bro&wse"), this);

    h1->addWidget(fileLabel);
    h1->addWidget(filepathEdit);
    h1->addWidget(browseButton1);
    h1->addWidget(gethashButton1);

    h2->addWidget(directoryLabel);
    h2->addWidget(dirpathEdit);
    h2->addWidget(browseButton2);
    h2->addWidget(gethashButton2);

    h3->addWidget(messageBox);

    gethashButton1->setEnabled(false);
    gethashButton2->setEnabled(false);

    messageBox->setReadOnly(true);

    setLayout(v1);

    connect(browseButton1, SIGNAL(clicked()), SLOT(getFileFromBrowser()));
    connect(browseButton2, SIGNAL(clicked()), SLOT(getDirectoryFromBrowser()));
    connect(filepathEdit, SIGNAL(textChanged(QString)), this, SLOT(validateForm(QString)));
    connect(dirpathEdit, SIGNAL(textChanged(QString)), this, SLOT(validateForm(QString)));
    connect(gethashButton1, SIGNAL(clicked()), SLOT(hashFile()));
    connect(gethashButton2, SIGNAL(clicked()), SLOT(hashDirectory()));

}

hashpanel::~hashpanel()
{
}


void hashpanel::getDirectoryFromBrowser()
{
    dirpathEdit->setText(QFileDialog::getExistingDirectory(this,tr("Find directory"),"/"));
}


void hashpanel::getFileFromBrowser()
{
    filepathEdit->setText(QFileDialog::getOpenFileName(this,tr("Find file"),"/","*", 0));
}

void hashpanel::validateForm(const QString&)
{
    int toggleDirPath = 0;
    int toggleFilePath = 0;

    if (((filepathEdit->text()).compare(tr(""))))
        toggleFilePath = 1;

    if (((dirpathEdit->text()).compare(tr(""))))
        toggleDirPath = 1;

    if ( toggleFilePath)
    {
        gethashButton1->setEnabled(true);
    }
    else
    {
        gethashButton1->setEnabled(false);
    }
    if ( toggleDirPath)
    {
        gethashButton2->setEnabled(true);
    }
    else
    {
        gethashButton2->setEnabled(false);
    }
}

void hashpanel::hashDirectory()
{
    QWidget::setCursor(QCursor(Qt::WaitCursor));
    browseButton2->setDisabled(true);
    gethashButton2->setDisabled(true);

    messageBox->clear();
    messageBox->parentWidget()->repaint();

    if (kcore::hashKDir(dirpathEdit->text().toStdString(), messageBox))
        messageBox->append("Err: Dir I/O.");

    browseButton2->setDisabled(false);
    gethashButton2->setDisabled(false);
    QWidget::setCursor(QCursor(Qt::PointingHandCursor));
}

void hashpanel::hashFile()
{
    QWidget::setCursor(QCursor(Qt::WaitCursor));
    browseButton1->setDisabled(true);
    gethashButton1->setDisabled(true);

    messageBox->clear();
    messageBox->parentWidget()->repaint();

    if (kcore::hashKFile(filepathEdit->text().toStdString(), messageBox))
        messageBox->append("Err: file I/O.");

    browseButton1->setDisabled(false);
    gethashButton1->setDisabled(false);
    QWidget::setCursor(QCursor(Qt::PointingHandCursor));
}

#if OS == 0x03
#include "hashpanel.moc"
#endif