/*
    $Id: kcore.cpp 298 2010-02-16 21:55:03Z irfangul82 $
    Simple file and directory encryption
    Copyright (C) 2010  Irfan Gulamali

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "kcore.h"

kcore::kcore()
{
}

kcore::~kcore()
{
}


int kcore::decryptKDir(std::string dir, std::string key, QTextEdit *msg)
{
    using namespace std;

    unsigned int ret;
    unsigned char cache_fname[PATH_MAX];
    ifstream dirParseFile;
    string fileParsed;
    char fileName[PATH_MAX]="";

    //encrypt dir
    parseDirDBClean();
    if (parseDirTree((unsigned char*)dir.c_str()))
    {
        msg->append("Error: failed to parse directory.");
        msg->parentWidget()->repaint();

        return FILEIO_ERROR;
    }

    dirParseFile.open(PARS_DIR_PATH);

    if (!(dirParseFile.is_open()))
    {
        msg->append("Error: failed to parse directory cache file .ko-parsedir in current path.");
        msg->parentWidget()->repaint();

        return FILEIO_ERROR;
    }

    while ( !(dirParseFile.eof()) )
    {
        dirParseFile.getline(fileName, PATH_MAX);
        fileParsed.assign(fileName);
        if (fileParsed.length()>0)
        {
            msg->append("decrypting.. ");
            msg->parentWidget()->repaint();
            ret = decryptKFile(fileParsed, key);
            if ( ret )
            {
                msg->insertPlainText("Err: ");
                msg->insertPlainText(QString(fileParsed.c_str()));
                msg->append(" ");
                msg->parentWidget()->repaint();
            }
            else
            {
                msg->insertPlainText(QString(fileParsed.c_str()));
                msg->append(" ");
                msg->parentWidget()->repaint();
            }
        }
    }

    dirParseFile.close();
	parseDirDBClean();

    return 0;
}

int kcore::decryptKFile(std::string file, std::string key)
{
    unsigned int ret;
    unsigned char fileStr[PATH_MAX] = "";
    unsigned char keyStr[34] = "";

    strcpy((char*)fileStr, file.c_str());
    strcpy((char*)keyStr, key.c_str());

    ret = decryptFile(fileStr, keyStr);

    return ret;
}

int kcore::encryptKDir(std::string dir, std::string key,  QTextEdit *msg)
{
    using namespace std;

    unsigned int ret;
    unsigned char cache_fname[PATH_MAX];
    ifstream dirParseFile;
    string fileParsed;
    char fileName[PATH_MAX]="";

    //encrypt dir
    parseDirDBClean();
    if (parseDirTree((unsigned char*)dir.c_str()))
    {
        msg->append("Error: failed to parse directory.");
        msg->parentWidget()->repaint();

        return FILEIO_ERROR;
    }

    dirParseFile.open(PARS_DIR_PATH);

    if (!(dirParseFile.is_open()))
    {
        msg->append("Error: failed to parse directory cache file .ko-parsedir in current path.");
        msg->parentWidget()->repaint();

        return FILEIO_ERROR;
    }

    while ( !(dirParseFile.eof()) )
    {
        dirParseFile.getline(fileName, PATH_MAX);
        fileParsed.assign(fileName);
        if (fileParsed.length()>0)
        {
            msg->append("encrypting.. ");
            msg->parentWidget()->repaint();
            ret = encryptKFile(fileParsed, key);
            if ( ret )
            {
                msg->insertPlainText("Err: ");
                msg->insertPlainText(QString(fileParsed.c_str()));
                msg->append(" ");
                msg->parentWidget()->repaint();
            }
            else
            {
                msg->insertPlainText(QString(fileParsed.c_str()));
                msg->append(" ");
                msg->parentWidget()->repaint();
            }
        }
    }

    dirParseFile.close();
	parseDirDBClean();
    return 0;
}

int kcore::encryptKFile(std::string file, std::string key)
{
    unsigned int ret;
    unsigned char fileStr[PATH_MAX] = "";
    unsigned char keyStr[34] = "";

    strcpy((char*)fileStr, file.c_str());
    strcpy((char*)keyStr, key.c_str());

    ret = encryptFile( fileStr, keyStr);

    return ret;
}

int kcore::encryptKMsg(std::string key, QTextEdit *msg)
{
    using namespace std;
    unsigned int ret;
    ofstream msgOut;
    ifstream msgIn;
    const int colsz = 60;
    unsigned char buff[colsz] ="";
    unsigned char msgFile[] = "msg.txt";
	unsigned char c_key[33] = "";
    std::string strbuff;

    msgOut.open((const char*)msgFile);
    if (!(msgOut.is_open()))
        return FILEIO_ERROR;

    msgOut.write((msg->toPlainText().toStdString().c_str()), (msg->toPlainText().length()));
    msgOut.close();

	strcpy((char*)c_key, key.c_str());

    ret = encMsg(&msgFile[0], c_key);
	memset(c_key, 0, 33);
	
	if(ret)
		return ret;
	
	msgIn.open((const char*)msgFile);
    if (!(msgIn.is_open()))
        return FILEIO_ERROR;

    msg->clear();
    msg->parentWidget()->repaint();
    memset(buff, 0, colsz);

    while (!(msgIn.eof()))
    {
        std::getline(msgIn,strbuff);
        msg->append(QString(strbuff.c_str()));
    }
    msg->parentWidget()->repaint();
    msgIn.close();
    remove((const char*)msgFile);
    return 0;

}
int kcore::decryptKMsg(std::string key, QTextEdit *msg)
{
    using namespace std;
    unsigned int ret;
    ofstream msgOut;
    ifstream msgIn;
    const int colsz = 60;
    unsigned char buff[colsz] ="";
    unsigned char msgFile[] = "msg.txt";
	unsigned char c_key[33] = "";
    std::string strbuff;

    msgOut.open((const char*)msgFile);
    if (!(msgOut.is_open()))
        return FILEIO_ERROR;

    msgOut.write((msg->toPlainText().toStdString().c_str()), (msg->toPlainText().length()));
    msgOut.close();

	strcpy((char*)c_key, key.c_str());
    ret = decMsg(&msgFile[0], c_key);
	memset(c_key, 0, 33);
	
	if(ret)
		return ret;

    msgIn.open((const char*)msgFile);
    if (!(msgIn.is_open()))
        return FILEIO_ERROR;

    msg->clear();
    msg->parentWidget()->repaint();

    while (!(msgIn.eof()))
    {
        std::getline(msgIn,strbuff);
        msg->append(QString(strbuff.c_str()));
    }
    msg->parentWidget()->repaint();
    msgIn.close();
    remove((const char*)msgFile);
    return 0;
}

int kcore::hashKDir(std::string dir, QTextEdit* msg)
{
    using namespace std;

    unsigned int ret =0;
    unsigned char hash[41] = "";
    string cache_fname;
    ifstream dirParseFile;

	parseDirDBClean();
    if (parseDirTree((unsigned char*)dir.c_str()))
    {
        msg->append("Error: failed to parse directory.");
        msg->parentWidget()->repaint();
        return FILEIO_ERROR;
    }

    dirParseFile.open(PARS_DIR_PATH);

    if (!(dirParseFile.is_open()))
    {
        msg->append("Error: failed to parse directory cache file .ko-parsedir in current path.");
        msg->parentWidget()->repaint();

        return FILEIO_ERROR;
    }

    while ( !(dirParseFile.eof()) )
    {
        std::getline(dirParseFile, cache_fname);
        if (cache_fname.length()>0)
        {
            ret = hashKFile(cache_fname,msg);
        }
    }
	dirParseFile.close();
	parseDirDBClean();
    return ret;
}

int kcore::hashKFile(std::string file, QTextEdit *msg)
{
    unsigned int ret;
    unsigned char hash[41] = "";

    ret = gtHash((unsigned char*)file.c_str(),hash);
    if (ret)
    {
        return FILEIO_ERROR;
    }
    msg->append(QString((const char*)hash));
    msg->insertPlainText("  ");
    msg->insertPlainText(QString(file.c_str()));
    msg->append("  ");
    msg->parentWidget()->repaint();

    return ret;
}

std::string kcore::errorMessage(unsigned int ret)
{
    switch (ret) {
    case 0:
        return std::string("Completed!");
        break;
    case FILEIO_ERROR:
        return std::string("Error: file/dir not found");
        break;
    case FILEFORMAT_ERROR:
        return std::string("Error: file format error encountered!");
        break;
    case FILESZ_OUTBOUND:
        return std::string("Error: file size is out of bound!");
        break;
    case SHA_ERR:
        return std::string("Error: decryption failed -- sha1 doesn't verify");
        break;
    case ENC_EXT:
        return std::string("Error: the .enc file extention is missing during decryption or not expected during encryption ");
        break;
    default:
        return std::string("Error: unrecognized error!");
    }
}





