/*
    $Id: messagepanel.cpp 298 2010-02-16 21:55:03Z irfangul82 $
    Simple file and directory encryption
    Copyright (C) 2010  Irfan Gulamali

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "messagepanel.h"
#include "kcore.h"

messagepanel::messagepanel(QWidget* parent) : QWidget(parent)
{
    messageLabel = new QLabel(tr("message"), this);
    passwordLabel = new QLabel(tr("password: "), this);
    encryptButton = new QPushButton("&Encrypt", this);
    decryptButton = new QPushButton("De&crypt", this);
    password1 = new QLineEdit(this);
    password2 = new QLineEdit(this);
    messageBox = new QTextEdit(this);

    password1->setEchoMode(QLineEdit::Password);
    password2->setEchoMode(QLineEdit::Password);

    encryptButton->setDisabled(true);
    decryptButton->setDisabled(true);

    v1 = new QVBoxLayout(this);
    h1 = new QHBoxLayout(this);
    h2 = new QHBoxLayout(this);
    h3 = new QHBoxLayout(this);
    h4 = new QHBoxLayout(this);

    v1->addItem(h1);
    v1->addItem(h2);
    v1->addItem(h3);
    v1->addItem(h4);

    h1->addWidget(passwordLabel);
    h1->addWidget(password1);
    h1->addWidget(password2);

    h2->addWidget(encryptButton);
    h2->addWidget(decryptButton);

    h3->addWidget(messageLabel);
    h4->addWidget(messageBox);

    password1->setMaxLength(32);
    password2->setMaxLength(32);

    setLayout(v1);

    connect(password1, SIGNAL(textEdited (const QString&)), this, SLOT(validateForm(const QString&)));
    connect(password2, SIGNAL(textEdited (const QString&)), this, SLOT(validateForm(const QString&)));
    connect(messageBox, SIGNAL(textChanged()), SLOT(validateForm_0()));
    connect(encryptButton, SIGNAL(clicked()), SLOT(encryptMessage()));
    connect(decryptButton, SIGNAL(clicked()), SLOT(decryptMessage()));

}

messagepanel::~messagepanel()
{

}


void messagepanel::validateForm(const QString& )
{
    int toggleButton = 0;
    int toggleText = 0;

    if (((messageBox->toPlainText()).compare(tr(""))))
        toggleText = 1;

    if (!((password1->text()).compare(password2->text())) &&
            (password1->text().compare(tr(""))) && (password2->text().compare(tr(""))))
        toggleButton = 1;

    if ( (toggleButton*toggleText) )
    {
        encryptButton->setDisabled(false);
        decryptButton->setDisabled(false);
    }
    else
    {
        encryptButton->setDisabled(true);
        decryptButton->setDisabled(true);
    }
}

void messagepanel::validateForm_0()
{
    int toggleButton = 0;
    int toggleText = 0;

    if (((messageBox->toPlainText()).compare(tr(""))))
        toggleText = 1;

    if (!((password1->text()).compare(password2->text())) &&
            (password1->text().compare(tr(""))) && (password2->text().compare(tr(""))))
        toggleButton = 1;

    if ( (toggleButton*toggleText) )
    {
        encryptButton->setDisabled(false);
        decryptButton->setDisabled(false);
    }
    else
    {
        encryptButton->setDisabled(true);
        decryptButton->setDisabled(true);
    }
}

void messagepanel::encryptMessage()
{
    QWidget::setCursor(QCursor(Qt::WaitCursor));
    kcore::encryptKMsg((password1->text().toStdString()), messageBox);
    QWidget::setCursor(QCursor(Qt::PointingHandCursor));
}

void messagepanel::decryptMessage()
{
    QWidget::setCursor(QCursor(Qt::WaitCursor));
    kcore::decryptKMsg((password1->text().toStdString()), messageBox);
    QWidget::setCursor(QCursor(Qt::PointingHandCursor));
}

#if OS == 0x03
#include "messagepanel.moc"
#endif